@@ -12,7 +12,7 @@
  *     https://opensource.org/licenses/BSD-3-Clause
  */
 
-#define _GNU_SOURCE
+#define _GNU_SOURCE /*strndup */
 
 #include <assert.h>
 #include <ctype.h>
@@ -20,8 +20,9 @@
 #include <limits.h>
 #include <pthread.h>
 #include <string.h>
+#include <stdarg.h>
+#include <stdio.h>
 #include <stdlib.h>
-#include <sys/syscall.h>
 #include <sys/types.h>
 #include <unistd.h>
 
@@ -36,12 +37,16 @@ THREAD_LOCAL int8_t ly_errno_glob;
 API LY_ERR *
 ly_errno_glob_address(void)
 {
+    FUN_IN;
+
     return (LY_ERR *)&ly_errno_glob;
 }
 
 API LY_VECODE
 ly_vecode(const struct ly_ctx *ctx)
 {
+    FUN_IN;
+
     struct ly_err_item *i;
 
     i = ly_err_first(ctx);
@@ -55,6 +60,8 @@ ly_vecode(const struct ly_ctx *ctx)
 API const char *
 ly_errmsg(const struct ly_ctx *ctx)
 {
+    FUN_IN;
+
     struct ly_err_item *i;
 
     i = ly_err_first(ctx);
@@ -68,6 +75,8 @@ ly_errmsg(const struct ly_ctx *ctx)
 API const char *
 ly_errpath(const struct ly_ctx *ctx)
 {
+    FUN_IN;
+
     struct ly_err_item *i;
 
     i = ly_err_first(ctx);
@@ -81,6 +90,8 @@ ly_errpath(const struct ly_ctx *ctx)
 API const char *
 ly_errapptag(const struct ly_ctx *ctx)
 {
+    FUN_IN;
+
     struct ly_err_item *i;
 
     i = ly_err_first(ctx);
@@ -94,6 +105,8 @@ ly_errapptag(const struct ly_ctx *ctx)
 API struct ly_err_item *
 ly_err_first(const struct ly_ctx *ctx)
 {
+    FUN_IN;
+
     if (!ctx) {
         return NULL;
     }
@@ -119,6 +132,8 @@ ly_err_free(void *ptr)
 API void
 ly_err_clean(struct ly_ctx *ctx, struct ly_err_item *eitem)
 {
+    FUN_IN;
+
     struct ly_err_item *i, *first;
 
     first = ly_err_first(ctx);
@@ -144,24 +159,6 @@ ly_err_clean(struct ly_ctx *ctx, struct ly_err_item *eitem)
     }
 }
 
-#ifndef  __USE_GNU
-
-char *
-get_current_dir_name(void)
-{
-    char tmp[PATH_MAX];
-    char *retval;
-
-    if (getcwd(tmp, sizeof(tmp))) {
-        retval = strdup(tmp);
-        LY_CHECK_ERR_RETURN(!retval, LOGMEM(NULL), NULL);
-        return retval;
-    }
-    return NULL;
-}
-
-#endif
-
 const char *
 strpbrk_backwards(const char *s, const char *accept, unsigned int s_len)
 {
@@ -257,8 +254,8 @@ transform_module_name2import_prefix(const struct lys_module *module, const char
 }
 
 static int
-_transform_json2xml_subexp(const struct lys_module *module, const char *expr, char **out, size_t *out_used, size_t *out_size, int schema, int inst_id, const char ***prefixes,
-                    const char ***namespaces, uint32_t *ns_count)
+_transform_json2xml_subexp(const struct lys_module *module, const char *expr, char **out, size_t *out_used, size_t *out_size,
+                           int schema, int inst_id, const char ***prefixes, const char ***namespaces, uint32_t *ns_count)
 {
     const char *cur_expr, *end, *prefix, *literal;
     char *name;
@@ -396,10 +393,6 @@ _transform_json2xml_subexp(const struct lys_module *module, const char *expr, ch
     return 0;
 
 error:
-    if (!schema && ns_count) {
-        free(*prefixes);
-        free(*namespaces);
-    }
     lyxp_expr_free(exp);
     return 1;
 }
@@ -436,6 +429,14 @@ _transform_json2xml(const struct lys_module *module, const char *expr, int schem
         return lydict_insert_zc(module->ctx, out);
     }
 
+    /* fail */
+    if (ns_count) {
+        *ns_count = 0;
+        free(*prefixes);
+        *prefixes = NULL;
+        free(*namespaces);
+        *namespaces = NULL;
+    }
     free(out);
     return NULL;
 }
@@ -601,6 +602,8 @@ transform_xml2json(struct ly_ctx *ctx, const char *expr, struct lyxml_elem *xml,
 API char *
 ly_path_xml2json(struct ly_ctx *ctx, const char *xml_path, struct lyxml_elem *xml)
 {
+    FUN_IN;
+
     const char *json_path;
     char *ret = NULL;
 
@@ -1190,6 +1193,8 @@ error:
 API char *
 ly_path_data2schema(struct ly_ctx *ctx, const char *data_path)
 {
+    FUN_IN;
+
     struct lyxp_expr *exp;
     uint16_t out_used, cur_exp = 0;
     char *out;
@@ -1287,12 +1292,6 @@ ly_new_node_validity(const struct lys_node *schema)
 
     validity = LYD_VAL_OK;
 
-    if (schema->nodetype & (LYS_LEAF | LYS_LEAFLIST)) {
-        if (((struct lys_node_leaf *)schema)->type.base == LY_TYPE_LEAFREF) {
-            /* leafref target validation */
-            validity |= LYD_VAL_LEAFREF;
-        }
-    }
     if (schema->nodetype & (LYS_LEAFLIST | LYS_LIST)) {
         /* duplicit instance check */
         validity |= LYD_VAL_DUP;
@@ -1426,3 +1425,41 @@ lyb_has_schema_model(struct lys_node *sibling, const struct lys_module **models,
 
     return 0;
 }
+
+/**
+ * @brief Static table of the UTF8 characters lengths according to their first byte.
+ */
+static const unsigned char
+utf8_char_length_table[] = {
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
+    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
+    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
+    4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 6, 6, 1, 1
+};
+
+/**
+ * @brief Use of utf8_char_length_table.
+ */
+#define UTF8LEN(x) utf8_char_length_table[((unsigned char)(x))]
+
+size_t
+ly_strlen_utf8(const char *str)
+{
+    size_t clen, len;
+    const char *ptr;
+
+    for (len = 0, clen = strlen(str), ptr = str; *ptr && len < clen; ++len, ptr += UTF8LEN(*ptr));
+    return len;
+}
