@@ -16,6 +16,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 #include <string.h>
 #include <assert.h>
 #include <inttypes.h>
@@ -35,7 +36,8 @@
 struct mlist {
     struct mlist *next;
     struct lys_module *module;
-} *mlist = NULL, *mlist_new;
+    uint8_t printed;
+};
 
 static int
 modlist_add(struct mlist **mlist, const struct lys_module *mod)
@@ -53,6 +55,7 @@ modlist_add(struct mlist **mlist, const struct lys_module *mod)
         LY_CHECK_ERR_RETURN(!iter, LOGMEM(mod->ctx), EXIT_FAILURE);
         iter->next = *mlist;
         iter->module = (struct lys_module *)mod;
+        iter->printed = 0;
         *mlist = iter;
     }
 
@@ -60,12 +63,22 @@ modlist_add(struct mlist **mlist, const struct lys_module *mod)
 }
 
 static void
-xml_print_ns(struct lyout *out, const struct lyd_node *node, int options)
+free_mlist(struct mlist **mlist) {
+    struct mlist *miter;
+    while (*mlist) {
+        miter = *mlist;
+        *mlist = miter->next;
+        free(miter);
+    }
+}
+
+static void
+xml_print_ns(struct lyout *out, const struct lyd_node *node, struct mlist **mlist, int options)
 {
     struct lyd_node *next, *cur, *node2;
     struct lyd_attr *attr;
     const struct lys_module *wdmod = NULL;
-    struct mlist *mlist = NULL, *miter;
+    struct mlist *miter;
     int r;
 
     assert(out);
@@ -79,7 +92,7 @@ xml_print_ns(struct lyout *out, const struct lyd_node *node, int options)
             /* exception for NETCONF's filter attributes */
             continue;
         } else {
-            r = modlist_add(&mlist, lys_main_module(attr->annotation->module));
+            r = modlist_add(mlist, lys_main_module(attr->annotation->module));
         }
         if (r) {
             goto print;
@@ -93,7 +106,7 @@ xml_print_ns(struct lyout *out, const struct lyd_node *node, int options)
         if (node->dflt && (options & (LYP_WD_ALL_TAG | LYP_WD_IMPL_TAG))) {
             /* get with-defaults module and print its namespace */
             wdmod = ly_ctx_get_module(node->schema->module->ctx, "ietf-netconf-with-defaults", NULL, 1);
-            if (wdmod && modlist_add(&mlist, wdmod)) {
+            if (wdmod && modlist_add(mlist, wdmod)) {
                 goto print;
             }
         }
@@ -106,7 +119,7 @@ xml_print_ns(struct lyout *out, const struct lyd_node *node, int options)
         if (options & (LYP_WD_ALL_TAG | LYP_WD_IMPL_TAG)) {
             /* get with-defaults module and print its namespace */
             wdmod = ly_ctx_get_module(node->schema->module->ctx, "ietf-netconf-with-defaults", NULL, 1);
-            if (wdmod && modlist_add(&mlist, wdmod)) {
+            if (wdmod && modlist_add(mlist, wdmod)) {
                 goto print;
             }
         }
@@ -120,7 +133,7 @@ xml_print_ns(struct lyout *out, const struct lyd_node *node, int options)
                         /* exception for NETCONF's filter attributes */
                         continue;
                     } else {
-                        r = modlist_add(&mlist, lys_main_module(attr->annotation->module));
+                        r = modlist_add(mlist, lys_main_module(attr->annotation->module));
                     }
                     if (r) {
                         goto print;
@@ -135,12 +148,14 @@ xml_print_ns(struct lyout *out, const struct lyd_node *node, int options)
 
 print:
     /* print used namespaces */
-    while (mlist) {
-        miter = mlist;
-        mlist = mlist->next;
+    miter = *mlist;
+    while (miter) {
 
-        ly_print(out, " xmlns:%s=\"%s\"", miter->module->prefix, miter->module->ns);
-        free(miter);
+        if (!miter->printed) {
+            ly_print(out, " xmlns:%s=\"%s\"", miter->module->prefix, miter->module->ns);
+            miter->printed = 1;
+        }
+        miter = miter->next;
     }
 }
 
@@ -156,6 +171,8 @@ xml_print_attrs(struct lyout *out, const struct lyd_node *node, int options)
     char *p;
     size_t len;
 
+    LY_PRINT_SET;
+
     /* with-defaults */
     if (node->schema->nodetype & (LYS_LEAF | LYS_LEAFLIST)) {
         if ((node->dflt && (options & (LYP_WD_ALL_TAG | LYP_WD_IMPL_TAG))) ||
@@ -183,7 +200,6 @@ xml_print_attrs(struct lyout *out, const struct lyd_node *node, int options)
                 xml_expr = transform_json2xml(node->schema->module, attr->value_str, 0, &prefs, &nss, &ns_count);
                 if (!xml_expr) {
                     /* error */
-                    ly_print(out, "\"(!error!)\"");
                     return EXIT_FAILURE;
                 }
 
@@ -240,7 +256,6 @@ printinst:
                                           &prefs, &nss, &ns_count);
             if (!xml_expr) {
                 /* error */
-                ly_print(out, "(!error!)");
                 return EXIT_FAILURE;
             }
 
@@ -260,7 +275,7 @@ printinst:
 
         default:
             /* error */
-            ly_print(out, "(!error!)");
+            LOGINT(node->schema->module->ctx);
             return EXIT_FAILURE;
         }
 
@@ -271,7 +286,7 @@ printinst:
         }
     }
 
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
@@ -288,6 +303,9 @@ xml_print_leaf(struct lyout *out, int level, const struct lyd_node *node, int to
     char *p;
     size_t len;
     enum int_log_opts prev_ilo;
+    struct mlist *mlist = NULL;
+
+    LY_PRINT_SET;
 
     if (toplevel || !node->parent || nscmp(node, node->parent)) {
         /* print "namespace" */
@@ -298,7 +316,8 @@ xml_print_leaf(struct lyout *out, int level, const struct lyd_node *node, int to
     }
 
     if (toplevel) {
-        xml_print_ns(out, node, options);
+        xml_print_ns(out, node, &mlist, options);
+        free_mlist(&mlist);
     }
 
     if (xml_print_attrs(out, node, options)) {
@@ -371,7 +390,6 @@ printvalue:
                                       &prefs, &nss, &ns_count);
         if (!xml_expr) {
             /* error */
-            ly_print(out, "\"(!error!)\"");
             return EXIT_FAILURE;
         }
 
@@ -401,7 +419,6 @@ printvalue:
             type = lyd_leaf_type((struct lyd_node_leaf_list *)leaf);
             if (!type) {
                 /* error */
-                ly_print(out, "\"(!error!)\"");
                 return EXIT_FAILURE;
             }
             datatype = type->base;
@@ -411,12 +428,14 @@ printvalue:
         goto printvalue;
 
     case LY_TYPE_EMPTY:
+    case LY_TYPE_UNKNOWN:
+        /* treat <edit-config> node without value as empty */
         ly_print(out, "/>");
         break;
 
     default:
         /* error */
-        ly_print(out, "\"(!error!)\"");
+        LOGINT(node->schema->module->ctx);
         return EXIT_FAILURE;
     }
 
@@ -424,7 +443,7 @@ printvalue:
         ly_print(out, "\n");
     }
 
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
@@ -432,6 +451,9 @@ xml_print_container(struct lyout *out, int level, const struct lyd_node *node, i
 {
     struct lyd_node *child;
     const char *ns;
+    struct mlist *mlist = NULL;
+
+    LY_PRINT_SET;
 
     if (toplevel || !node->parent || nscmp(node, node->parent)) {
         /* print "namespace" */
@@ -442,7 +464,8 @@ xml_print_container(struct lyout *out, int level, const struct lyd_node *node, i
     }
 
     if (toplevel) {
-        xml_print_ns(out, node, options);
+        xml_print_ns(out, node, &mlist, options);
+        free_mlist(&mlist);
     }
 
     if (xml_print_attrs(out, node, options)) {
@@ -451,7 +474,7 @@ xml_print_container(struct lyout *out, int level, const struct lyd_node *node, i
 
     if (!node->child) {
         ly_print(out, "/>%s", level ? "\n" : "");
-        return EXIT_SUCCESS;
+        goto finish;
     }
     ly_print(out, ">%s", level ? "\n" : "");
 
@@ -463,7 +486,8 @@ xml_print_container(struct lyout *out, int level, const struct lyd_node *node, i
 
     ly_print(out, "%*s</%s>%s", LEVEL, INDENT, node->schema->name, level ? "\n" : "");
 
-    return EXIT_SUCCESS;
+finish:
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
@@ -471,6 +495,9 @@ xml_print_list(struct lyout *out, int level, const struct lyd_node *node, int is
 {
     struct lyd_node *child;
     const char *ns;
+    struct mlist *mlist = NULL;
+
+    LY_PRINT_SET;
 
     if (is_list) {
         /* list print */
@@ -483,7 +510,8 @@ xml_print_list(struct lyout *out, int level, const struct lyd_node *node, int is
         }
 
         if (toplevel) {
-            xml_print_ns(out, node, options);
+            xml_print_ns(out, node, &mlist, options);
+            free_mlist(&mlist);
         }
         if (xml_print_attrs(out, node, options)) {
             return EXIT_FAILURE;
@@ -491,7 +519,7 @@ xml_print_list(struct lyout *out, int level, const struct lyd_node *node, int is
 
         if (!node->child) {
             ly_print(out, "/>%s", level ? "\n" : "");
-            return EXIT_SUCCESS;
+            goto finish;
         }
         ly_print(out, ">%s", level ? "\n" : "");
 
@@ -507,7 +535,8 @@ xml_print_list(struct lyout *out, int level, const struct lyd_node *node, int is
         xml_print_leaf(out, level, node, toplevel, options);
     }
 
-    return EXIT_SUCCESS;
+finish:
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 static int
@@ -517,6 +546,9 @@ xml_print_anydata(struct lyout *out, int level, const struct lyd_node *node, int
     struct lyd_node_anydata *any = (struct lyd_node_anydata *)node;
     struct lyd_node *iter;
     const char *ns;
+    struct mlist *mlist = NULL;
+
+    LY_PRINT_SET;
 
     if (toplevel || !node->parent || nscmp(node, node->parent)) {
         /* print "namespace" */
@@ -527,7 +559,7 @@ xml_print_anydata(struct lyout *out, int level, const struct lyd_node *node, int
     }
 
     if (toplevel) {
-        xml_print_ns(out, node, options);
+        xml_print_ns(out, node, &mlist, options);
     }
     if (xml_print_attrs(out, node, options)) {
         return EXIT_FAILURE;
@@ -536,14 +568,27 @@ xml_print_anydata(struct lyout *out, int level, const struct lyd_node *node, int
         /* no content */
         ly_print(out, "/>%s", level ? "\n" : "");
     } else {
+        if (any->value_type == LYD_ANYDATA_LYB) {
+            /* parse into a data tree */
+            ly_errno = 0;
+            iter = lyd_parse_mem(node->schema->module->ctx, any->value.mem, LYD_LYB, LYD_OPT_DATA | LYD_OPT_STRICT
+                                 | LYD_OPT_TRUSTED);
+            if (!ly_errno) {
+                /* successfully parsed */
+                free(any->value.mem);
+                any->value_type = LYD_ANYDATA_DATATREE;
+                any->value.tree = iter;
+            }
+        }
         if (any->value_type == LYD_ANYDATA_DATATREE) {
             /* print namespaces in the anydata data tree */
             LY_TREE_FOR(any->value.tree, iter) {
-                xml_print_ns(out, iter, options);
+                xml_print_ns(out, iter, &mlist, options);
             }
         }
         /* close opening tag ... */
         ly_print(out, ">");
+        free_mlist(&mlist);
         /* ... and print anydata content */
         switch (any->value_type) {
         case LYD_ANYDATA_CONSTSTRING:
@@ -573,7 +618,7 @@ xml_print_anydata(struct lyout *out, int level, const struct lyd_node *node, int
             break;
         case LYD_ANYDATA_JSON:
         case LYD_ANYDATA_LYB:
-            /* JSON and LYB format is not supported */
+            /* JSON format is not supported (LYB failed to be converted) */
             LOGWRN(node->schema->module->ctx, "Unable to print anydata content (type %d) as XML.", any->value_type);
             break;
         case LYD_ANYDATA_STRING:
@@ -589,7 +634,7 @@ xml_print_anydata(struct lyout *out, int level, const struct lyd_node *node, int
         ly_print(out, "</%s>%s", node->schema->name, level ? "\n" : "");
     }
 
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(node->schema->module->ctx);
 }
 
 int
@@ -597,7 +642,7 @@ xml_print_node(struct lyout *out, int level, const struct lyd_node *node, int to
 {
     int ret = EXIT_SUCCESS;
 
-    if (!lyd_wd_toprint(node, options)) {
+    if (!lyd_toprint(node, options)) {
         /* wd says do not print */
         return EXIT_SUCCESS;
     }
@@ -638,11 +683,13 @@ xml_print_data(struct lyout *out, const struct lyd_node *root, int options)
     struct lys_node *parent = NULL;
     int level, action_input = 0;
 
+    LY_PRINT_SET;
+
     if (!root) {
         if (out->type == LYOUT_MEMORY || out->type == LYOUT_CALLBACK) {
             ly_print(out, "");
         }
-        return EXIT_SUCCESS;
+        goto finish;
     }
 
     level = (options & LYP_FORMAT ? 1 : 0);
@@ -675,7 +722,7 @@ xml_print_data(struct lyout *out, const struct lyd_node *root, int options)
     }
 
     if (action_input) {
-        ly_print(out, "%*s<action xmlns=\"urn:ietf:params:xml:ns:yang:1\">%s", LEVEL, INDENT, level ? "\n" : "");
+        ly_print(out, "%*s<action xmlns=\"%s\">%s", LEVEL, INDENT, LY_NSYANG, level ? "\n" : "");
         if (level) {
             ++level;
         }
@@ -698,8 +745,9 @@ xml_print_data(struct lyout *out, const struct lyd_node *root, int options)
         ly_print(out, "%*s</action>%s", LEVEL, INDENT, level ? "\n" : "");
     }
 
+finish:
     ly_print_flush(out);
 
-    return EXIT_SUCCESS;
+    LY_PRINT_RET(NULL);
 }
 
